/****************************************************************************
**
** Copyright (C) 2020 MikroElektronika d.o.o.
** Contact: https://www.mikroe.com/contact
**
** This file is part of the mikroSDK package
**
** Commercial License Usage
**
** Licensees holding valid commercial NECTO compilers AI licenses may use this
** file in accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The mikroElektronika Company.
** For licensing terms and conditions see
** https://www.mikroe.com/legal/software-license-agreement.
** For further information use the contact form at
** https://www.mikroe.com/contact.
**
**
** GNU Lesser General Public License Usage
**
** Alternatively, this file may be used for
** non-commercial projects under the terms of the GNU Lesser
** General Public License version 3 as published by the Free Software
** Foundation: https://www.gnu.org/licenses/lgpl-3.0.html.
**
** The above copyright notice and this permission notice shall be
** included in all copies or substantial portions of the Software.
**
** THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
** OF MERCHANTABILITY, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
** TO THE WARRANTIES FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
** IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
** DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT
** OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
** OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
**
****************************************************************************/

// ------------------------------------------------------------------ INCLUDES
#include "board.h"
#include "drv_i2c_master.h"

// If hardware without mikroBUS socket is used, make sure to define adequate pin.
#ifndef MIKROBUS_1_SCL
#define MIKROBUS_1_SCL NC
#endif

// If hardware without mikroBUS socket is used, make sure to define adequate pin.
#ifndef MIKROBUS_1_SDA
#define MIKROBUS_1_SDA NC
#endif

#define I2C_MASTER_TIMEOUT 1000 // I2C number of retries for each operation.
#define I2C_MASTER_SLAVE_ADDRESS 0x3C // I2C slave address.

// ------------------------------------------------------------------ VARIABLES
static i2c_master_t i2c_master; // I2C driver context structure.
static i2c_master_config_t i2c_master_cfg; // I2C driver configuration structure.

// ------------------------------------------------------------------ USER CODE
void main()
{
    uint8_t *write_data_buf; // I2C data transmit buffer.
    uint8_t *read_data_buf; // I2C data receive buffer.

    // Initialize I2C configuration structure to default values.
    i2c_master_configure_default( &i2c_master_cfg );

    i2c_master_cfg.scl = MIKROBUS_1_SCL; // Specify desired SCL pin.
    i2c_master_cfg.sda = MIKROBUS_1_SDA; // Specify desired SDA pin.

    // Allocate resources for I2C module.
    if ( i2c_master_open( &i2c_master, &i2c_master_cfg ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Set transmission rate.
    if ( i2c_master_set_speed( &i2c_master, I2C_MASTER_SPEED_STANDARD ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Set timeout value.
    if ( i2c_master_set_timeout( &i2c_master, I2C_MASTER_TIMEOUT ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Set slave address. Make sure to set adequate slave address for hardware used.
    if ( i2c_master_set_slave_address( &i2c_master, I2C_MASTER_SLAVE_ADDRESS ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Write data to the I2C bus.
    if ( i2c_master_write( &i2c_master, &write_data_buf, sizeof( write_data_buf ) ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Read data from the I2C bus.
    if ( i2c_master_read( &i2c_master, &read_data_buf, sizeof( read_data_buf ) ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Write data to I2C bus followed by read.
    if ( i2c_master_write_then_read( &i2c_master,  &write_data_buf, sizeof( write_data_buf ),
                                                   &read_data_buf, sizeof( read_data_buf )
                                                   ) == I2C_MASTER_ERROR )
    {
        // Error handling.
        return;
    }

    // Close the I2C module object handler.
    i2c_master_close( &i2c_master );
}
